@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.12.2.3 2002/03/12 15:12:11 srittau Exp $
+ * $Id: main.c,v 1.26 2009-10-14 02:24:05 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -12,35 +12,28 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
 #include <signal.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/param.h>
 #include <sys/uio.h>
-#include <sys/syslog.h>
+#include <atalk/logger.h>
 #include <sys/time.h>
 #include <sys/socket.h>
 
 #include <errno.h>
 
-#include <netatalk/endian.h>
+#include <atalk/adouble.h>
+
 #include <netatalk/at.h>
 #include <atalk/compat.h>
 #include <atalk/dsi.h>
 #include <atalk/atp.h>
 #include <atalk/asp.h>
 #include <atalk/afp.h>
-#include <atalk/adouble.h>
 #include <atalk/paths.h>
 #include <atalk/util.h>
 #include <atalk/server_child.h>
+#include <atalk/server_ipc.h>
 
 #include "globals.h"
 #include "afp_config.h"
@@ -57,21 +50,13 @@ static int argc = 0;
 static char **argv = NULL;
 #endif /* TRU64 */
 
-#ifdef DID_MTAB
-#include "parse_mtab.h"
-#endif /* DID_MTAB */
-
 unsigned char	nologin = 0;
 
-#ifdef DID_MTAB
-/* global mount table; afpd_st_cnid uses this to lookup the right entry.  */
-static struct afpd_mount_table *afpd_mount_table = NULL;
-#endif /* DID_MTAB */
-
 struct afp_options default_options;
 static AFPConfig *configs;
 static server_child *server_children;
 static fd_set save_rfds;
+static int    Ipc_fd = -1;
 
 #ifdef TRU64
 void afp_get_cmdline( int *ac, char ***av)
@@ -87,45 +72,68 @@ static void afp_exit(const int i)
     exit(i);
 }
 
+/* ------------------
+   initialize fd set we are waiting for.
+*/
+static void set_fd(int ipc_fd)
+{
+    AFPConfig   *config;
+
+    FD_ZERO(&save_rfds);
+    for (config = configs; config; config = config->next) {
+        if (config->fd < 0) /* for proxies */
+            continue;
+        FD_SET(config->fd, &save_rfds);
+    }
+    if (ipc_fd >= 0) {
+        FD_SET(ipc_fd, &save_rfds);
+    }
+}
+ 
+/* ------------------ */
 static void afp_goaway(int sig)
 {
+
 #ifndef NO_DDP
     asp_kill(sig);
 #endif /* ! NO_DDP */
+
     dsi_kill(sig);
     switch( sig ) {
     case SIGTERM :
-        syslog( LOG_INFO, "shutting down on signal %d", sig );
+        LOG(log_info, logtype_afpd, "shutting down on signal %d", sig );
         break;
+    case SIGUSR1 :
     case SIGHUP :
         /* w/ a configuration file, we can force a re-read if we want */
         nologin++;
-        if ((nologin + 1) & 1) {
+        auth_unload();
+        if (sig == SIGHUP || ((nologin + 1) & 1)) {
             AFPConfig *config;
 
-            syslog(LOG_INFO, "re-reading configuration file");
+            LOG(log_info, logtype_afpd, "re-reading configuration file");
             for (config = configs; config; config = config->next)
                 if (config->server_cleanup)
                     config->server_cleanup(config);
 
+            /* configfree close atp socket used for DDP tickle, there's an issue
+             * with atp tid.
+            */
             configfree(configs, NULL);
             if (!(configs = configinit(&default_options))) {
-                syslog(LOG_ERR, "config re-read: no servers configured");
-                afp_exit(1);
-            }
-            FD_ZERO(&save_rfds);
-            for (config = configs; config; config = config->next) {
-                if (config->fd < 0)
-                    continue;
-                FD_SET(config->fd, &save_rfds);
+                LOG(log_error, logtype_afpd, "config re-read: no servers configured");
+                afp_exit(EXITERR_CONF);
             }
+            set_fd(Ipc_fd);
         } else {
-            syslog(LOG_INFO, "disallowing logins");
-            auth_unload();
+            LOG(log_info, logtype_afpd, "disallowing logins");
+        }
+        if (sig == SIGHUP) {
+            nologin = 0;
         }
         break;
     default :
-        syslog( LOG_ERR, "afp_goaway: bad signal" );
+        LOG(log_error, logtype_afpd, "afp_goaway: bad signal" );
     }
     if ( sig == SIGTERM ) {
         AFPConfig *config;
@@ -139,19 +147,19 @@ static void afp_goaway(int sig)
     return;
 }
 
-static void child_handler()
+static void child_handler(int sig _U_)
 {
     server_child_handler(server_children);
 }
 
-int main( ac, av )
-int		ac;
-char	**av;
+int main(int ac, char **av)
 {
     AFPConfig           *config;
     fd_set              rfds;
+    void                *ipc;
     struct sigaction	sv;
     sigset_t            sigs;
+    int                 ret;
 
 #ifdef TRU64
     argc = ac;
@@ -159,57 +167,96 @@ char	**av;
     set_auth_parameters( ac, av );
 #endif /* TRU64 */
 
+#ifdef DEBUG1
+    fault_setup(NULL);
+#endif
     afp_options_init(&default_options);
     if (!afp_options_parse(ac, av, &default_options))
-        exit(1);
+        exit(EXITERR_CONF);
 
-    umask( default_options.umask );
+    /* Save the user's current umask for use with CNID (and maybe some 
+     * other things, too). */
+    default_options.save_mask = umask( default_options.umask );
 
     switch(server_lock("afpd", default_options.pidfile,
                        default_options.flags & OPTION_DEBUG)) {
     case -1: /* error */
-        exit(1);
+        exit(EXITERR_SYS);
     case 0: /* child */
         break;
     default: /* server */
         exit(0);
     }
 
-#ifdef DID_MTAB
-    /* if we are going to use afpd.mtab, load the file */
-    afpd_mount_table = afpd_mtab_parse ( AFPD_MTAB_FILE );
-#endif /* DID_MTAB */
+#if 0
+    /* Register CNID  */
+    cnid_init();
+#endif
 
     /* install child handler for asp and dsi. we do this before afp_goaway
      * as afp_goaway references stuff from here. 
      * XXX: this should really be setup after the initial connections. */
     if (!(server_children = server_child_alloc(default_options.connections,
                             CHILD_NFORKS))) {
-        syslog( LOG_ERR, "main: server_child alloc: %s", strerror(errno) );
-        afp_exit(1);
+        LOG(log_error, logtype_afpd, "main: server_child alloc: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
     }
-
+    
+#ifdef AFP3x
+    /* linux at least up to 2.4.22 send a SIGXFZ for vfat fs,
+       even if the file is open with O_LARGEFILE ! */
+#ifdef SIGXFSZ
+    signal(SIGXFSZ , SIG_IGN); 
+#endif
+#endif    
+    
     memset(&sv, 0, sizeof(sv));
     sv.sa_handler = child_handler;
     sigemptyset( &sv.sa_mask );
+    sigaddset(&sv.sa_mask, SIGALRM);
+    sigaddset(&sv.sa_mask, SIGHUP);
+    sigaddset(&sv.sa_mask, SIGTERM);
+    sigaddset(&sv.sa_mask, SIGUSR1);
+    
     sv.sa_flags = SA_RESTART;
-    if ( sigaction( SIGCHLD, &sv, 0 ) < 0 ) {
-        syslog( LOG_ERR, "main: sigaction: %s", strerror(errno) );
-        afp_exit(1);
+    if ( sigaction( SIGCHLD, &sv, NULL ) < 0 ) {
+        LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
     }
 
     sv.sa_handler = afp_goaway;
     sigemptyset( &sv.sa_mask );
-    sigaddset(&sv.sa_mask, SIGHUP);
+    sigaddset(&sv.sa_mask, SIGALRM);
     sigaddset(&sv.sa_mask, SIGTERM);
+    sigaddset(&sv.sa_mask, SIGHUP);
+    sigaddset(&sv.sa_mask, SIGCHLD);
     sv.sa_flags = SA_RESTART;
-    if ( sigaction( SIGHUP, &sv, 0 ) < 0 ) {
-        syslog( LOG_ERR, "main: sigaction: %s", strerror(errno) );
-        afp_exit(1);
+    if ( sigaction( SIGUSR1, &sv, NULL ) < 0 ) {
+        LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
     }
-    if ( sigaction( SIGTERM, &sv, 0 ) < 0 ) {
-        syslog( LOG_ERR, "main: sigaction: %s", strerror(errno) );
-        afp_exit(1);
+
+    sigemptyset( &sv.sa_mask );
+    sigaddset(&sv.sa_mask, SIGALRM);
+    sigaddset(&sv.sa_mask, SIGTERM);
+    sigaddset(&sv.sa_mask, SIGUSR1);
+    sigaddset(&sv.sa_mask, SIGCHLD);
+    sv.sa_flags = SA_RESTART;
+    if ( sigaction( SIGHUP, &sv, NULL ) < 0 ) {
+        LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
+    }
+
+
+    sigemptyset( &sv.sa_mask );
+    sigaddset(&sv.sa_mask, SIGALRM);
+    sigaddset(&sv.sa_mask, SIGHUP);
+    sigaddset(&sv.sa_mask, SIGUSR1);
+    sigaddset(&sv.sa_mask, SIGCHLD);
+    sv.sa_flags = SA_RESTART;
+    if ( sigaction( SIGTERM, &sv, NULL ) < 0 ) {
+        LOG(log_error, logtype_afpd, "main: sigaction: %s", strerror(errno) );
+        afp_exit(EXITERR_SYS);
     }
 
     /* afpd.conf: not in config file: lockfile, connections, configfile
@@ -221,22 +268,30 @@ char	**av;
      */
 
     sigemptyset(&sigs);
+    sigaddset(&sigs, SIGALRM);
     sigaddset(&sigs, SIGHUP);
+    sigaddset(&sigs, SIGUSR1);
+#if 0
+    /* don't block SIGTERM */
     sigaddset(&sigs, SIGTERM);
+#endif
+    sigaddset(&sigs, SIGCHLD);
+
     sigprocmask(SIG_BLOCK, &sigs, NULL);
     if (!(configs = configinit(&default_options))) {
-        syslog(LOG_ERR, "main: no servers configured: %s\n", strerror(errno));
-        afp_exit(1);
+        LOG(log_error, logtype_afpd, "main: no servers configured");
+        afp_exit(EXITERR_CONF);
     }
     sigprocmask(SIG_UNBLOCK, &sigs, NULL);
 
-    /* watch atp and dsi sockets. */
-    FD_ZERO(&save_rfds);
-    for (config = configs; config; config = config->next) {
-        if (config->fd < 0) /* for proxies */
-            continue;
-        FD_SET(config->fd, &save_rfds);
+    /* Register CNID  */
+    cnid_init();
+
+    /* watch atp, dsi sockets and ipc parent/child file descriptor. */
+    if ((ipc = server_ipc_create())) {
+        Ipc_fd = server_ipc_parent(ipc);
     }
+    set_fd(Ipc_fd);
 
     /* wait for an appleshare connection. parent remains in the loop
      * while the children get handled by afp_over_{asp,dsi}.  this is
@@ -246,18 +301,24 @@ char	**av;
      * solution. */
     while (1) {
         rfds = save_rfds;
-        if (select(FD_SETSIZE, &rfds, NULL, NULL, NULL) < 0) {
+        sigprocmask(SIG_UNBLOCK, &sigs, NULL);
+        ret = select(FD_SETSIZE, &rfds, NULL, NULL, NULL);
+        sigprocmask(SIG_BLOCK, &sigs, NULL);
+        if (ret < 0) {
             if (errno == EINTR)
                 continue;
-            syslog(LOG_ERR, "main: can't wait for input: %s", strerror(errno));
+            LOG(log_error, logtype_afpd, "main: can't wait for input: %s", strerror(errno));
             break;
         }
-
+        if (Ipc_fd >=0 && FD_ISSET(Ipc_fd, &rfds)) {
+            server_ipc_read(server_children);
+        }
         for (config = configs; config; config = config->next) {
             if (config->fd < 0)
                 continue;
-            if (FD_ISSET(config->fd, &rfds))
+            if (FD_ISSET(config->fd, &rfds)) {
                 config->server_start(config, configs, server_children);
+            }
         }
     }
 
