@@ -1,5 +1,5 @@
 /*
- * $Id: nbp.c,v 1.5.2.2 2002/02/07 23:58:46 srittau Exp $
+ * $Id: nbp.c,v 1.13 2009-10-14 02:24:05 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved. See COPYRIGHT.
@@ -12,7 +12,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/syslog.h>
+#include <atalk/logger.h>
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/socket.h>
@@ -46,11 +46,7 @@ extern int	transition;
 struct nbptab	*nbptab = NULL;
 
 static 
-void nbp_ack( fd, nh_op, nh_id, to )
-    int			fd;
-    int			nh_op;
-    int			nh_id;
-    struct sockaddr_at	*to;
+void nbp_ack( int fd, int nh_op, int nh_id, struct sockaddr_at *to)
 {
     struct nbphdr	nh;
     char		*data, packet[ SZ_NBPHDR + 1 ];
@@ -64,22 +60,18 @@ void nbp_ack( fd, nh_op, nh_id, to )
     data += SZ_NBPHDR;
     if ( sendto( fd, packet, data - packet, 0, (struct sockaddr *)to,
 	    sizeof( struct sockaddr_at )) < 0 ) {
-	syslog( LOG_ERR, "sendto: %s", strerror(errno) );
+	LOG(log_error, logtype_atalkd, "sendto: %s", strerror(errno) );
     }
 }
 
-int nbp_packet( ap, from, data, len )
-    struct atport	*ap;
-    struct sockaddr_at	*from;
-    char		*data;
-    int			len;
+int nbp_packet(struct atport *ap, struct sockaddr_at *from, char *data, int len)
 {
     struct nbphdr	nh;
     struct nbptuple	nt;
     struct nbpnve	nn;
     struct sockaddr_at	sat;
     struct nbptab	*ntab;
-    struct ziptab	*zt;
+    struct ziptab	*zt=NULL;
     struct interface	*iface;
     struct list		*l;
     struct rtmptab	*rtmp;
@@ -89,23 +81,23 @@ int nbp_packet( ap, from, data, len )
 
     end = data + len;
     if ( data >= end ) {
-	syslog( LOG_INFO, "nbp_packet malformed packet" );
+	LOG(log_info, logtype_atalkd, "nbp_packet malformed packet" );
 	return 1;
     }
     if ( *data++ != DDPTYPE_NBP ) {
-	syslog( LOG_INFO, "nbp_packet bad ddp type" );
+	LOG(log_info, logtype_atalkd, "nbp_packet bad ddp type" );
 	return 1;
     }
 
     if ( data + SZ_NBPHDR + SZ_NBPTUPLE > end ) {
-	syslog( LOG_INFO, "nbp_packet: malformed packet" );
+	LOG(log_info, logtype_atalkd, "nbp_packet: malformed packet" );
 	return 1;
     }
     memcpy( &nh, data, SZ_NBPHDR );
     nbpop = data;			/* remember for fwd and brrq */
     data += SZ_NBPHDR;
     if ( nh.nh_cnt != 1 ) {
-	syslog( LOG_INFO, "nbp_packet: bad tuple count (%d/%d)", nh.nh_cnt,
+	LOG(log_info, logtype_atalkd, "nbp_packet: bad tuple count (%d/%d)", nh.nh_cnt,
 		nh.nh_op );
 	return 1;
     }
@@ -125,7 +117,7 @@ int nbp_packet( ap, from, data, len )
     /* object */
     tmplen = (u_char) *data;
     if ( data >= end || tmplen > 32 || data + tmplen > end ) {
-	syslog( LOG_INFO, "nbp_packet: malformed packet" );
+	LOG(log_info, logtype_atalkd, "nbp_packet: malformed packet" );
 	return 1;
     }
     nn.nn_objlen = tmplen;
@@ -136,7 +128,7 @@ int nbp_packet( ap, from, data, len )
     /* type */
     tmplen = (u_char) *data;
     if ( data >= end || tmplen > 32 || data + tmplen > end ) {
-	syslog( LOG_INFO, "nbp_packet: malformed packet" );
+	LOG(log_info, logtype_atalkd, "nbp_packet: malformed packet" );
 	return 1;
     }
     nn.nn_typelen = tmplen;
@@ -147,7 +139,7 @@ int nbp_packet( ap, from, data, len )
     /* zone */
     tmplen = (u_char) *data;
     if ( data >= end || tmplen > 32 || data + tmplen > end ) {
-	syslog( LOG_INFO, "nbp_packet: malformed packet" );
+	LOG(log_info, logtype_atalkd, "nbp_packet: malformed packet" );
 	return 1;
     }
     zonep = data;			/* remember for fwd */
@@ -157,7 +149,7 @@ int nbp_packet( ap, from, data, len )
     data += nn.nn_zonelen;
 
     if ( data != end ) {
-	syslog( LOG_INFO, "nbp_packet: malformed packet" );
+	LOG(log_info, logtype_atalkd, "nbp_packet: malformed packet" );
 	return 1;
     }
 
@@ -173,7 +165,7 @@ int nbp_packet( ap, from, data, len )
 	    if ( interfaces->i_next->i_rt->rt_zt ) {
 		zt = (struct ziptab *)interfaces->i_next->i_rt->rt_zt->l_data;
 	    } else {
-		zt = 0;
+		zt = NULL;
 	    }
 	} else {
 	    for ( zt = ziptab; zt; zt = zt->zt_next ) {
@@ -182,7 +174,7 @@ int nbp_packet( ap, from, data, len )
 		    break;
 		}
 	    }
-	    if ( zt == 0 ) {
+	    if ( zt == NULL ) {
 		nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
 		return 0;
 	    }
@@ -192,7 +184,7 @@ int nbp_packet( ap, from, data, len )
 	 * Observe that we don't have to do any local-zone verification
 	 * if the zone aleady has a multicast address set.
 	 */
-	if ( zt != 0 && zt->zt_bcast == 0 ) {
+	if ( zt != NULL && zt->zt_bcast == NULL ) {
 	    /*
 	     * Check if zone is associated with any of our local interfaces.
 	     */
@@ -202,18 +194,18 @@ int nbp_packet( ap, from, data, len )
 			break;
 		    }
 		}
-		if ( l != 0 ) {
+		if ( l != NULL ) {
 		    break;
 		}
 	    }
-	    if ( iface == 0 ) {
+	    if ( iface == NULL ) {
 		nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
 		return 0;
 	    }
 
 	    /* calculate and save multicast address */
 	    if (zone_bcast(zt) < 0) {
-	        syslog(LOG_ERR, "nbp_packet: zone_bcast");
+	        LOG(log_error, logtype_atalkd, "nbp_packet: zone_bcast");
 		return -1;
 	    }
 
@@ -225,7 +217,7 @@ int nbp_packet( ap, from, data, len )
 		    if ( zt == (struct ziptab *)l->l_data ) {
 		        /* add multicast */
 		        if (addmulti(iface->i_name, zt->zt_bcast) < 0) {
-			    syslog( LOG_ERR, "nbp_packet: addmulti: %s",
+			    LOG(log_error, logtype_atalkd, "nbp_packet: addmulti: %s",
 				    strerror(errno) );
 			    return -1;
 			}
@@ -235,14 +227,14 @@ int nbp_packet( ap, from, data, len )
 	}
 
 	if (( ntab = (struct nbptab *)malloc( sizeof( struct nbptab )))
-		== 0 ) {
-	    syslog( LOG_ERR, "nbp_packet: malloc: %s", strerror(errno) );
+		== NULL ) {
+	    LOG(log_error, logtype_atalkd, "nbp_packet: malloc: %s", strerror(errno) );
 	    return -1;
 	}
 	memcpy( &ntab->nt_nve, &nn, sizeof( struct nbpnve ));
 	ntab->nt_iface = ap->ap_iface;
 	ntab->nt_next = nbptab;
-	ntab->nt_prev = 0;
+	ntab->nt_prev = NULL;
 	if ( nbptab ) {
 	    nbptab->nt_prev = ntab;
 	}
@@ -305,15 +297,15 @@ int nbp_packet( ap, from, data, len )
 		break;
 	    }
 	}
-	if ( ntab == 0 ) {
+	if ( ntab == NULL ) {
 	    nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
 	    return 0;
 	}
 
-	if ( ntab->nt_next != 0 ) {
+	if ( ntab->nt_next != NULL ) {
 	    ntab->nt_next->nt_prev = ntab->nt_prev;
 	}
-	if ( ntab->nt_prev != 0 ) {
+	if ( ntab->nt_prev != NULL ) {
 	    ntab->nt_prev->nt_next = ntab->nt_next;
 	}
 	if ( ntab == nbptab ) {
@@ -370,7 +362,7 @@ int nbp_packet( ap, from, data, len )
 		    break;
 		}
 	    }
-	    if ( zt == 0 ) {
+	    if ( zt == NULL ) {
 		nbp_ack( ap->ap_fd, NBPOP_ERROR, (int)nh.nh_id, from );
 		return 0;
 	    }
@@ -380,7 +372,7 @@ int nbp_packet( ap, from, data, len )
 	 * If we've got no zones, send out LKUP on the local net.
 	 * Otherwise, look through the zone table.
 	 */
-	if ( zt == 0 ) {
+	if ( zt == NULL ) {
 #ifdef BSD4_4
 	    sat.sat_len = sizeof( struct sockaddr_at );
 #endif /* BSD4_4 */
@@ -400,7 +392,7 @@ int nbp_packet( ap, from, data, len )
 		    break;
 		}
 	    }
-	    if ( iface == 0 ) {
+	    if ( iface == NULL ) {
 		return 0;
 	    }
 	    for ( ap = iface->i_ports; ap; ap = ap->ap_next ) {
@@ -411,7 +403,7 @@ int nbp_packet( ap, from, data, len )
 
 	    if ( sendto( ap->ap_fd, data - len, len, 0, (struct sockaddr *)&sat,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "nbp brrq sendto: %s", strerror(errno) );
+		LOG(log_error, logtype_atalkd, "nbp brrq sendto: %s", strerror(errno) );
 	    }
 
 	    locallkup = 1;
@@ -424,7 +416,7 @@ int nbp_packet( ap, from, data, len )
 	    for ( l = zt->zt_rt; l; l = l->l_next ) {
 		rtmp = (struct rtmptab *)l->l_data;
 
-		if ( rtmp->rt_gate == 0 ) {
+		if ( rtmp->rt_gate == NULL ) {
 		    for ( iface = interfaces; iface;
 			    iface = iface->i_next ) {
 			if ( iface->i_rt == rtmp ) {
@@ -432,7 +424,7 @@ int nbp_packet( ap, from, data, len )
 			}
 		    }
 		    if ( !iface ) {
-			syslog( LOG_ERR, "nbp_packet: \
+			LOG(log_error, logtype_atalkd, "nbp_packet: \
 Can't find route's interface!" );
 			return -1;
 		    }
@@ -446,13 +438,13 @@ Can't find route's interface!" );
 		    }
 		}
 		if ( !ap ) {
-		    syslog( LOG_ERR, "nbp_packet: Can't find port!" );
+		    LOG(log_error, logtype_atalkd, "nbp_packet: Can't find port!" );
 		    return -1;
 		}
 
 		if ( transition &&
 			( rtmp->rt_flags & RTMPTAB_EXTENDED ) == 0 ) {
-		    if ( rtmp->rt_gate == 0 ) {
+		    if ( rtmp->rt_gate == NULL ) {
 			locallkup = 1;
 		    }
 		    nh.nh_op = NBPOP_LKUP;
@@ -460,7 +452,7 @@ Can't find route's interface!" );
 		    sat.sat_addr.s_net = rtmp->rt_firstnet;
 		    sat.sat_addr.s_node = ATADDR_BCAST;
 		} else {
-		    if ( rtmp->rt_gate == 0 ) {
+		    if ( rtmp->rt_gate == NULL ) {
 			nh.nh_op = NBPOP_LKUP;
 			memcpy( nbpop, &nh, SZ_NBPHDR );
 			sat.sat_addr.s_net = 0;
@@ -477,7 +469,7 @@ Can't find route's interface!" );
 		if ( sendto( ap->ap_fd, data - len, len, 0,
 			(struct sockaddr *)&sat,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "nbp brrq sendto %u.%u: %s",
+		    LOG(log_error, logtype_atalkd, "nbp brrq sendto %u.%u: %s",
 			    ntohs( sat.sat_addr.s_net ), sat.sat_addr.s_node,
 			    strerror(errno) );
 		    continue;
@@ -499,7 +491,7 @@ Can't find route's interface!" );
 	    from->sat_addr.s_node = ATADDR_BCAST;
 	    if ( sendto( ap->ap_fd, data - len, len, 0, (struct sockaddr *)from,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "nbp fwd sendto %u.%u: %s",
+		LOG(log_error, logtype_atalkd, "nbp fwd sendto %u.%u: %s",
 			ntohs( from->sat_addr.s_net ), from->sat_addr.s_node,
 			strerror(errno) );
 		return 0;
@@ -563,7 +555,7 @@ Can't find route's interface!" );
 	     * Another tuple won't fit. Send what we've already
 	     * got, and start the next packet.
 	     */
-	    if ( data + SZ_NBPTUPLE + 3 + ntab->nt_nve.nn_objlen +
+	    if ( n > 14 || data + SZ_NBPTUPLE + 3 + ntab->nt_nve.nn_objlen +
 		    ntab->nt_nve.nn_typelen + ntab->nt_nve.nn_zonelen > end ) {
 		nh.nh_op = NBPOP_LKUPREPLY;
 		nh.nh_cnt = n;
@@ -575,7 +567,7 @@ Can't find route's interface!" );
 		if ( sendto( ap->ap_fd, packet, cc, 0,
 			(struct sockaddr *)&nn.nn_sat,
 			sizeof( struct sockaddr_at )) < 0 ) {
-		    syslog( LOG_ERR, "nbp lkup sendto %u.%u: %s",
+		    LOG(log_error, logtype_atalkd, "nbp lkup sendto %u.%u: %s",
 			    ntohs( nn.nn_sat.sat_addr.s_net ),
 			    nn.nn_sat.sat_addr.s_node,
 			    strerror(errno) );
@@ -638,7 +630,7 @@ Can't find route's interface!" );
 	    if ( sendto( ap->ap_fd, packet, cc, 0,
 		    (struct sockaddr *)&nn.nn_sat,
 		    sizeof( struct sockaddr_at )) < 0 ) {
-		syslog( LOG_ERR, "nbp lkup sendto %u.%u: %s",
+		LOG(log_error, logtype_atalkd, "nbp lkup sendto %u.%u: %s",
 			ntohs( nn.nn_sat.sat_addr.s_net ),
 			nn.nn_sat.sat_addr.s_node,
 			strerror(errno) );
@@ -648,7 +640,7 @@ Can't find route's interface!" );
 	break;
 
     default :
-	syslog( LOG_INFO, "nbp_packet: bad op (%d)", nh.nh_op );
+	LOG(log_info, logtype_atalkd, "nbp_packet: bad op (%d)", nh.nh_op );
 	return 1;
     }
 
