@@ -1,5 +1,5 @@
 /*
- * $Id: cnid_dbd.c,v 1.1.4.18 2004/04/24 18:13:27 didg Exp $
+ * $Id: cnid_dbd.c,v 1.17 2010-03-31 09:47:32 franklahm Exp $
  *
  * Copyright (C) Joerg Lenneis 2003
  * All Rights Reserved.  See COPYING.
@@ -33,8 +33,8 @@
 #include <arpa/inet.h>
 #include <errno.h>
 #include <netdb.h>
-#include <time.h>    
- 
+#include <time.h>
+
 #include <netatalk/endian.h>
 #include <atalk/logger.h>
 #include <atalk/adouble.h>
@@ -46,133 +46,118 @@
 #define SOL_TCP IPPROTO_TCP
 #endif /* ! SOL_TCP */
 
-static void RQST_RESET(struct cnid_dbd_rqst  *r) 
-{ 
-   memset(r, 0, sizeof(struct cnid_dbd_rqst ));
+static void RQST_RESET(struct cnid_dbd_rqst  *r)
+{
+    memset(r, 0, sizeof(struct cnid_dbd_rqst ));
 }
 
 /* ----------- */
-extern char             Cnid_srv[MAXHOSTNAMELEN + 1];
-extern int              Cnid_port;
+#define MAX_DELAY 10
 
-static int tsock_getfd(char *host, int port)
+/* *MUST* be < afp tickle or it's never triggered (got EINTR first) */
+#define SOCK_DELAY 11
+
+static void delay(int sec)
 {
-int sock;
-struct sockaddr_in server;
-struct hostent* hp;
-int attr;
-int err;
+    struct timeval tv;
 
-    server.sin_family=AF_INET;
-    server.sin_port=htons((unsigned short)port);
-    if (!host) {
-        LOG(log_error, logtype_cnid, "getfd: -cnidserver not defined");
+    tv.tv_usec = 0;
+    tv.tv_sec  = sec;
+    select(0, NULL, NULL, NULL, &tv);
+}
+
+static int tsock_getfd(const char *host, const char *port)
+{
+    int sock = -1;
+    struct timeval tv;
+    int attr;
+    int err;
+    struct addrinfo hints, *servinfo, *p;
+
+    /* Prepare hint for getaddrinfo */
+    memset(&hints, 0, sizeof hints);
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_NUMERICSERV;
+
+    if ((err = getaddrinfo(host, port, &hints, &servinfo)) != 0) {
+        LOG(log_error, logtype_default, "tsock_getfd: getaddrinfo: CNID server %s:%s : %s\n", host, port, gai_strerror(err));
         return -1;
     }
-    
-    hp=gethostbyname(host);
-    if (!hp) {
-	unsigned long int addr=inet_addr(host);
-	LOG(log_warning, logtype_cnid, "getfd: Could not resolve host %s, trying numeric address instead", host);
-        if (addr!= (unsigned)-1)
-            hp=gethostbyaddr((char*)addr,sizeof(addr),AF_INET);
- 
-    	if (!hp) {
-	    LOG(log_error, logtype_cnid, "getfd: Could not resolve host %s", host);
-    	    return(-1);
-    	}
-    }
-    memcpy((char*)&server.sin_addr,(char*)hp->h_addr,sizeof(server.sin_addr));
-    sock=socket(PF_INET,SOCK_STREAM,0);
-    if (sock==-1) {
-	LOG(log_error, logtype_cnid, "getfd: socket %s: %s", host, strerror(errno));
-    	return(-1);
-    }
-    attr = 1;
-    if (setsockopt(sock, SOL_TCP, TCP_NODELAY, &attr, sizeof(attr)) == -1) {
-	LOG(log_error, logtype_cnid, "getfd: set TCP_NODELAY %s: %s", host, strerror(errno));
-	close(sock);
-	return(-1);
-    }
-    if(connect(sock ,(struct sockaddr*)&server,sizeof(server))==-1) {
-        struct timeval tv;
-        err = errno;
-    	close(sock);
-    	sock=-1;
-	LOG(log_error, logtype_cnid, "getfd: connect %s: %s", host, strerror(err));
-        switch (err) {
-        case ENETUNREACH:
-        case ECONNREFUSED: 
-            
-            tv.tv_usec = 0;
-            tv.tv_sec  = 5;
-            select(0, NULL, NULL, NULL, &tv);
-            break;
+
+    /* loop through all the results and bind to the first we can */
+    for (p = servinfo; p != NULL; p = p->ai_next) {
+        if ((sock = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) == -1) {
+            LOG(log_info, logtype_default, "tsock_getfd: socket CNID server %s:: %s", host, strerror(errno));
+                continue;
         }
+
+        attr = 1;
+        if (setsockopt(sock, SOL_TCP, TCP_NODELAY, &attr, sizeof(attr)) == -1) {
+            LOG(log_error, logtype_cnid, "getfd: set TCP_NODELAY CNID server %s: %s", host, strerror(errno));
+            close(sock);
+            continue;
+        }
+        
+        tv.tv_sec = SOCK_DELAY;
+        tv.tv_usec = 0;
+        if (setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0) {
+            LOG(log_error, logtype_cnid, "getfd: set SO_RCVTIMEO CNID server %s: %s", host, strerror(errno));
+            close(sock);
+            continue;
+        }
+
+        tv.tv_sec = SOCK_DELAY;
+        tv.tv_usec = 0;
+        if (setsockopt(sock, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv)) < 0) {
+            LOG(log_error, logtype_cnid, "getfd: set SO_SNDTIMEO CNID server %s: %s", host, strerror(errno));
+            close(sock);
+            continue;
+        }
+        
+        if (connect(sock, p->ai_addr, p->ai_addrlen) == -1) {
+            err = errno;
+            close(sock);
+            sock=-1;
+            LOG(log_error, logtype_cnid, "getfd: connect CNID server %s: %s", host, strerror(err));
+            continue;
+        }
+        
+        /* We've got a socket */
+        break;
     }
+
+    freeaddrinfo(servinfo);
+
+    if (p == NULL) {
+        LOG(log_error, logtype_cnid, "tsock_getfd: no suitable network config from CNID server %s:%s", host, port);
+        return -1;
+    }
+
     return(sock);
 }
 
 /* --------------------- */
-static int init_tsock(CNID_private *db)
+static int write_vec(int fd, struct iovec *iov, size_t towrite)
 {
-    int fd;
-    int len;
-    
-    if ((fd = tsock_getfd(Cnid_srv, Cnid_port)) < 0) 
-        return -1;
-    len = strlen(db->db_dir);
-    if (write(fd, &len, sizeof(int)) != sizeof(int)) {
-        LOG(log_error, logtype_cnid, "init_tsock: Error/short write: %s", strerror(errno));
-        close(fd);
-        return -1;
-    }
-    if (write(fd, db->db_dir, len) != len) {
-        LOG(log_error, logtype_cnid, "init_tsock: Error/short write dir: %s", strerror(errno));
-        close(fd);
-        return -1;
-    }
-    return fd;
-}
-
-/* --------------------- */
-static int send_packet(CNID_private *db, struct cnid_dbd_rqst *rqst, int silent)
-{
-    struct iovec iov[2];
-    size_t towrite;
     ssize_t len;
-  
-    iov[0].iov_base = rqst;
-    iov[0].iov_len  = sizeof(struct cnid_dbd_rqst);
-    
-    if (!rqst->namelen) {
-        if (write(db->fd, rqst, sizeof(struct cnid_dbd_rqst)) != sizeof(struct cnid_dbd_rqst)) {
-	    if (!silent)
-		LOG(log_warning, logtype_cnid, "send_packet: Error/short write rqst (db_dir %s): %s", 
-		    db->db_dir, strerror(errno));
-            return -1;
-        }
-        return 0;
-    }
-    iov[1].iov_base = rqst->name;
-    iov[1].iov_len  = rqst->namelen;
+    size_t len1;
 
-    towrite = sizeof(struct cnid_dbd_rqst) +rqst->namelen;
+    LOG(log_maxdebug, logtype_cnid, "write_vec: request to write %d bytes", towrite);
+
+    len1 =  iov[1].iov_len;
     while (towrite > 0) {
-        if (((len = writev(db->fd, iov, 2)) == -1 && errno == EINTR) || !len)
+        if (((len = writev(fd, iov, 2)) == -1 && errno == EINTR) || !len)
             continue;
- 
-        if (len == towrite) /* wrote everything out */
+
+        if ((size_t)len == towrite) /* wrote everything out */
             break;
         else if (len < 0) { /* error */
-	    if (!silent)
-		LOG(log_warning, logtype_cnid, "send_packet: Error writev rqst (db_dir %s): %s", 
-		    db->db_dir, strerror(errno));
             return -1;
         }
- 
+
         towrite -= len;
-        if (towrite > rqst->namelen) { /* skip part of header */
+        if (towrite > len1) { /* skip part of header */
             iov[0].iov_base = (char *) iov[0].iov_base + len;
             iov[0].iov_len -= len;
         } else { /* skip to data */
@@ -184,6 +169,77 @@ static int send_packet(CNID_private *db, struct cnid_dbd_rqst *rqst, int silent)
             iov[1].iov_len -= len;
         }
     }
+
+    LOG(log_maxdebug, logtype_cnid, "write_vec: wrote %d bytes", towrite);
+
+    return 0;
+}
+
+/* --------------------- */
+static int init_tsock(CNID_private *db)
+{
+    int fd;
+    int len;
+    struct iovec iov[2];
+
+    LOG(log_debug, logtype_cnid, "init_tsock: BEGIN. Opening volume '%s', CNID Server: %s/%s", 
+        db->db_dir, db->cnidserver, db->cnidport);
+
+    if ((fd = tsock_getfd(db->cnidserver, db->cnidport)) < 0)
+        return -1;
+
+    len = strlen(db->db_dir);
+
+    iov[0].iov_base = &len;
+    iov[0].iov_len  = sizeof(int);
+
+    iov[1].iov_base = db->db_dir;
+    iov[1].iov_len  = len;
+
+    if (write_vec(fd, iov, len + sizeof(int)) < 0) {
+        LOG(log_error, logtype_cnid, "init_tsock: Error/short write: %s", strerror(errno));
+        close(fd);
+        return -1;
+    }
+
+    LOG(log_debug, logtype_cnid, "init_tsock: ok");
+
+    return fd;
+}
+
+/* --------------------- */
+static int send_packet(CNID_private *db, struct cnid_dbd_rqst *rqst)
+{
+    struct iovec iov[2];
+    size_t towrite;
+
+    LOG(log_maxdebug, logtype_cnid, "send_packet: BEGIN");
+
+    if (!rqst->namelen) {
+        if (write(db->fd, rqst, sizeof(struct cnid_dbd_rqst)) != sizeof(struct cnid_dbd_rqst)) {
+            LOG(log_warning, logtype_cnid, "send_packet: Error/short write rqst (db_dir %s): %s",
+                db->db_dir, strerror(errno));
+            return -1;
+        }
+        LOG(log_maxdebug, logtype_cnid, "send_packet: OK");
+        return 0;
+    }
+
+    iov[0].iov_base = rqst;
+    iov[0].iov_len  = sizeof(struct cnid_dbd_rqst);
+
+    iov[1].iov_base = rqst->name;
+    iov[1].iov_len  = rqst->namelen;
+
+    towrite = sizeof(struct cnid_dbd_rqst) +rqst->namelen;
+
+    if (write_vec(db->fd, iov, towrite) < 0) {
+        LOG(log_warning, logtype_cnid, "send_packet: Error writev rqst (db_dir %s): %s",
+            db->db_dir, strerror(errno));
+        return -1;
+    }
+    
+    LOG(log_maxdebug, logtype_cnid, "send_packet: OK");
     return 0;
 }
 
@@ -210,138 +266,146 @@ static int dbd_reply_stamp(struct cnid_dbd_rply *rply)
     return 0;
 }
 
-/* --------------------- 
+/* ------------------- */
+static ssize_t dbd_read(int socket, void *data, const size_t length)
+{
+    size_t stored;
+    ssize_t len;
+  
+    stored = 0;
+    while (stored < length) {
+        len = read(socket, (u_int8_t *) data + stored, length - stored);
+        if (len == -1) {
+            if (errno == EINTR)
+                continue;
+            return -1;
+        }
+        else if (len > 0)
+            stored += len;
+        else
+            break;
+    }
+    return stored;
+}
+
+/* ---------------------
  * send a request and get reply
  * assume send is non blocking
  * if no answer after sometime (at least MAX_DELAY secondes) return an error
-*/
-#define MAX_DELAY 40
-static int dbd_rpc(CNID_private *db, struct cnid_dbd_rqst *rqst, struct cnid_dbd_rply *rply, int silent)
+ */
+static int dbd_rpc(CNID_private *db, struct cnid_dbd_rqst *rqst, struct cnid_dbd_rply *rply)
 {
-    int  ret;
+    ssize_t ret;
     char *nametmp;
-    struct timeval tv;
-    fd_set readfds;
-    int    maxfd;
     size_t len;
 
-    if (send_packet(db, rqst, silent) < 0) {
+    LOG(log_maxdebug, logtype_cnid, "dbd_rpc: BEGIN");
+
+    if (send_packet(db, rqst) < 0) {
         return -1;
     }
-    FD_ZERO(&readfds);
-    FD_SET(db->fd, &readfds);
-    maxfd = db->fd +1;
-        
-    tv.tv_usec = 0;
-    tv.tv_sec  = MAX_DELAY;
-    while ((ret = select(maxfd + 1, &readfds, NULL, NULL, &tv)) < 0 && errno == EINTR);
-
-    if (ret < 0) {
-	if (!silent)
-	    LOG(log_error, logtype_cnid, "dbd_rpc: Error in select (db_dir %s): %s",
-		db->db_dir, strerror(errno));
-        return ret;
-    }
-    /* signal ? */
-    if (!ret) {
-        /* no answer */
-	if (!silent)
-	    LOG(log_error, logtype_cnid, "dbd_rpc: select timed out (db_dir %s)",
-		db->db_dir);
-        return -1;
-    }
-
     len = rply->namelen;
     nametmp = rply->name;
-    /* assume that if we have something then everything is there (doesn't sleep) */ 
-    if ((ret = read(db->fd, rply, sizeof(struct cnid_dbd_rply))) != sizeof(struct cnid_dbd_rply)) {
-	if (!silent)
-	    LOG(log_error, logtype_cnid, "dbd_rpc: Error reading header from fd (db_dir %s): %s",
-                db->db_dir, ret == -1?strerror(errno):"closed");
+
+    ret = dbd_read(db->fd, rply, sizeof(struct cnid_dbd_rply));
+
+    if (ret != sizeof(struct cnid_dbd_rply)) {
+        LOG(log_error, logtype_cnid, "dbd_rpc: Error reading header from fd (db_dir %s): %s",
+            db->db_dir, ret == -1?strerror(errno):"closed");
         rply->name = nametmp;
         return -1;
     }
     rply->name = nametmp;
     if (rply->namelen && rply->namelen > len) {
-	if (!silent)
-            LOG(log_error, logtype_cnid, 
-                 "dbd_rpc: Error reading name (db_dir %s): %s name too long wanted %d only %d, garbage?",
-                 db->db_dir, rply->namelen, len);
+        LOG(log_error, logtype_cnid,
+            "dbd_rpc: Error reading name (db_dir %s): %s name too long: %d. only wanted %d, garbage?",
+            db->db_dir, rply->name, rply->namelen, len);
         return -1;
     }
-    if (rply->namelen && (ret = read(db->fd, rply->name, rply->namelen)) != rply->namelen) {
-	if (!silent)
-            LOG(log_error, logtype_cnid, "dbd_rpc: Error reading name from fd (db_dir %s): %s",
-                db->db_dir, ret == -1?strerror(errno):"closed");
+    if (rply->namelen && (ret = dbd_read(db->fd, rply->name, rply->namelen)) != (ssize_t)rply->namelen) {
+        LOG(log_error, logtype_cnid, "dbd_rpc: Error reading name from fd (db_dir %s): %s",
+            db->db_dir, ret == -1?strerror(errno):"closed");
         return -1;
     }
+
+    LOG(log_maxdebug, logtype_cnid, "dbd_rpc: END");
+
     return 0;
 }
 
 /* -------------------- */
 static int transmit(CNID_private *db, struct cnid_dbd_rqst *rqst, struct cnid_dbd_rply *rply)
 {
-    struct timeval tv;
     time_t orig, t;
-    int silent = 1;
-    
+    int clean = 1; /* no errors so far - to prevent sleep on first try */
+
+    LOG(log_debug7, logtype_cnid, "transmit: BEGIN");
+
     if (db->changed) {
         /* volume and db don't have the same timestamp
-        */
+         */
         return -1;
     }
-    time(&orig);
     while (1) {
-
         if (db->fd == -1) {
+            struct cnid_dbd_rqst rqst_stamp;
+            struct cnid_dbd_rply rply_stamp;
+            char  stamp[ADEDLEN_PRIVSYN];
+
+            LOG(log_debug, logtype_cnid, "transmit: connecting to cnid_dbd ...");
             if ((db->fd = init_tsock(db)) < 0) {
-                time(&t);
-                if (t - orig > MAX_DELAY)
-                    return -1;
-	        continue;
+                goto transmit_fail;
             }
+            dbd_initstamp(&rqst_stamp);
+            memset(stamp, 0, ADEDLEN_PRIVSYN);
+            rply_stamp.name = stamp;
+            rply_stamp.namelen = ADEDLEN_PRIVSYN;
+
+            if (dbd_rpc(db, &rqst_stamp, &rply_stamp) < 0)
+                goto transmit_fail;
+            if (dbd_reply_stamp(&rply_stamp ) < 0)
+                goto transmit_fail;
+
             if (db->notfirst) {
-                struct cnid_dbd_rqst rqst_stamp;
-                struct cnid_dbd_rply rply_stamp;
-                char  stamp[ADEDLEN_PRIVSYN];
-                
-                dbd_initstamp(&rqst_stamp);
-        	memset(stamp, 0, ADEDLEN_PRIVSYN);
-                rply_stamp.name = stamp;
-                rply_stamp.namelen = ADEDLEN_PRIVSYN;
-                
-        	if (dbd_rpc(db, &rqst_stamp, &rply_stamp, silent) < 0)
-        	    goto transmit_fail;
-        	if (dbd_reply_stamp(&rply_stamp ) < 0)
-        	    goto transmit_fail;
-        	if (memcmp(stamp, db->stamp, ADEDLEN_PRIVSYN)) {
-                    LOG(log_error, logtype_cnid, "transmit: not the same db!");
-        	    db->changed = 1;
-        	    return -1;
-        	}
+                LOG(log_debug7, logtype_cnid, "transmit: reconnected to cnid_dbd, comparing database stamps...");
+                if (memcmp(stamp, db->stamp, ADEDLEN_PRIVSYN)) {
+                    LOG(log_error, logtype_cnid, "transmit: ... not the same db!");
+                    db->changed = 1;
+                    return -1;
+                }
+                LOG(log_debug7, logtype_cnid, "transmit: ... OK.");
             }
-
+            else {
+                db->notfirst = 1;
+                if (db->client_stamp)
+                    memcpy(db->client_stamp, stamp, ADEDLEN_PRIVSYN);
+                memcpy(db->stamp, stamp, ADEDLEN_PRIVSYN);
+            }
+            LOG(log_debug, logtype_cnid, "transmit: succesfully attached to cnid_dbd for volume '%s' with stamp '%08lx'.", 
+                db->db_dir, *(uint64_t *)stamp);
         }
-        if (!dbd_rpc(db, rqst, rply, silent))
+        if (!dbd_rpc(db, rqst, rply)) {
+            LOG(log_debug7, logtype_cnid, "transmit: END OK");
             return 0;
-
-transmit_fail:
-	silent = 0; /* From now on dbd_rpc and subroutines called from there
-                       will log messages if something goes wrong again */
+        }
+    transmit_fail:
         if (db->fd != -1) {
             close(db->fd);
+            db->fd = -1; /* FD not valid... will need to reconnect */
         }
-        time(&t);
-        if (t - orig > MAX_DELAY) {
-	    LOG(log_error, logtype_cnid, "transmit: Request to dbd daemon (db_dir %s) timed out.", db->db_dir);
-            return -1;
-	}
 
-        /* sleep a little before retry */
-        db->fd = -1;
-        tv.tv_usec = 0;
-        tv.tv_sec  = 5;
-        select(0, NULL, NULL, NULL, &tv);
+        if (!clean) { /* don't sleep if just got disconnected by cnid server */
+            time(&t);
+            if (t - orig > MAX_DELAY) {
+                LOG(log_error, logtype_cnid, "transmit: Request to dbd daemon (db_dir %s) timed out.", db->db_dir);
+                return -1;
+            }
+            /* sleep a little before retry */
+            delay(2);
+        } else {
+            clean = 0; /* false... next time sleep */
+            time(&orig);
+        }
     }
     return -1;
 }
@@ -350,17 +414,17 @@ transmit_fail:
 static struct _cnid_db *cnid_dbd_new(const char *volpath)
 {
     struct _cnid_db *cdb;
-    
+
     if ((cdb = (struct _cnid_db *)calloc(1, sizeof(struct _cnid_db))) == NULL)
-	return NULL;
-	
+        return NULL;
+
     if ((cdb->volpath = strdup(volpath)) == NULL) {
         free(cdb);
-	return NULL;
+        return NULL;
     }
-    
-    cdb->flags = CNID_FLAG_PERSISTENT;
-    
+
+    cdb->flags = CNID_FLAG_PERSISTENT | CNID_FLAG_LAZY_INIT;
+
     cdb->cnid_add = cnid_dbd_add;
     cdb->cnid_delete = cnid_dbd_delete;
     cdb->cnid_get = cnid_dbd_get;
@@ -369,53 +433,56 @@ static struct _cnid_db *cnid_dbd_new(const char *volpath)
     cdb->cnid_resolve = cnid_dbd_resolve;
     cdb->cnid_getstamp = cnid_dbd_getstamp;
     cdb->cnid_update = cnid_dbd_update;
+    cdb->cnid_rebuild_add = cnid_dbd_rebuild_add;
     cdb->cnid_close = cnid_dbd_close;
-    
+
     return cdb;
 }
 
 /* ---------------------- */
-struct _cnid_db *cnid_dbd_open(const char *dir, mode_t mask)
+struct _cnid_db *cnid_dbd_open(struct cnid_open_args *args)
 {
     CNID_private *db = NULL;
     struct _cnid_db *cdb = NULL;
 
-    if (!dir) {
-         return NULL;
+    if (!args->dir) {
+        return NULL;
     }
-    
-    if ((cdb = cnid_dbd_new(dir)) == NULL) {
-        LOG(log_error, logtype_default, "cnid_open: Unable to allocate memory for database");
-	return NULL;
+
+    if ((cdb = cnid_dbd_new(args->dir)) == NULL) {
+        LOG(log_error, logtype_cnid, "cnid_open: Unable to allocate memory for database");
+        return NULL;
     }
-        
+
     if ((db = (CNID_private *)calloc(1, sizeof(CNID_private))) == NULL) {
         LOG(log_error, logtype_cnid, "cnid_open: Unable to allocate memory for database");
         goto cnid_dbd_open_fail;
     }
-    
+
     cdb->_private = db;
 
     /* We keep a copy of the directory in the db structure so that we can
        transparently reconnect later. */
-    strcpy(db->db_dir, dir);
+    strcpy(db->db_dir, args->dir);
     db->magic = CNID_DB_MAGIC;
     db->fd = -1;
-#ifdef DEBUG
-    LOG(log_info, logtype_cnid, "opening database connection to %s", db->db_dir); 
-#endif
+    db->cnidserver = strdup(args->cnidserver);
+    db->cnidport = strdup(args->cnidport);
+
+    LOG(log_debug, logtype_cnid, "cnid_dbd_open: Finished initializing cnid dbd module for volume '%s'", db->db_dir);
+
     return cdb;
 
 cnid_dbd_open_fail:
     if (cdb != NULL) {
-	if (cdb->volpath != NULL) {
-	    free(cdb->volpath);
-	}
-	free(cdb);
+        if (cdb->volpath != NULL) {
+            free(cdb->volpath);
+        }
+        free(cdb);
     }
     if (db != NULL)
-	free(db);
-	
+        free(db);
+
     return NULL;
 }
 
@@ -425,29 +492,28 @@ void cnid_dbd_close(struct _cnid_db *cdb)
     CNID_private *db;
 
     if (!cdb) {
-        LOG(log_error, logtype_afpd, "cnid_close called with NULL argument !");
-	return;
+        LOG(log_error, logtype_cnid, "cnid_close called with NULL argument !");
+        return;
     }
 
     if ((db = cdb->_private) != NULL) {
-#ifdef DEBUG 
-        LOG(log_info, logtype_cnid, "closing database connection to %s", db->db_dir);
-#endif  
-	if (db->fd >= 0)
-	    close(db->fd);
+        LOG(log_debug, logtype_cnid, "closing database connection for volume '%s'", db->db_dir);
+
+        if (db->fd >= 0)
+            close(db->fd);
         free(db);
     }
-    
+
     free(cdb->volpath);
     free(cdb);
-    
+
     return;
 }
 
 /* ---------------------- */
 cnid_t cnid_dbd_add(struct _cnid_db *cdb, const struct stat *st,
-                const cnid_t did, const char *name, const int len,
-                cnid_t hint)
+                    const cnid_t did, char *name, const size_t len,
+                    cnid_t hint)
 {
     CNID_private *db;
     struct cnid_dbd_rqst rqst;
@@ -475,18 +541,24 @@ cnid_t cnid_dbd_add(struct _cnid_db *cdb, const struct stat *st,
 
     rqst.ino = st->st_ino;
     rqst.type = S_ISDIR(st->st_mode)?1:0;
+    rqst.cnid = hint;
     rqst.did = did;
     rqst.name = name;
     rqst.namelen = len;
 
+    LOG(log_debug, logtype_cnid, "cnid_dbd_add: CNID: %u, name: '%s', inode: 0x%llx, type: %d (0=file, 1=dir)",
+        ntohl(did), name, (long long)st->st_ino, rqst.type);
+
+    rply.namelen = 0;
     if (transmit(db, &rqst, &rply) < 0) {
         errno = CNID_ERR_DB;
         return CNID_INVALID;
     }
-    
+
     switch(rply.result) {
     case CNID_DBD_RES_OK:
         id = rply.cnid;
+        LOG(log_debug, logtype_cnid, "cnid_dbd_add: got CNID: %u", ntohl(id));
         break;
     case CNID_DBD_RES_ERR_MAX:
         errno = CNID_ERR_MAX;
@@ -500,45 +572,48 @@ cnid_t cnid_dbd_add(struct _cnid_db *cdb, const struct stat *st,
     default:
         abort();
     }
+
     return id;
 }
 
 /* ---------------------- */
-cnid_t cnid_dbd_get(struct _cnid_db *cdb, const cnid_t did, const char *name,
-                const int len)
+cnid_t cnid_dbd_get(struct _cnid_db *cdb, const cnid_t did, char *name, const size_t len)
 {
     CNID_private *db;
     struct cnid_dbd_rqst rqst;
     struct cnid_dbd_rply rply;
     cnid_t id;
 
-
     if (!cdb || !(db = cdb->_private) || !name) {
-        LOG(log_error, logtype_cnid, "cnid_get: Parameter error");
-        errno = CNID_ERR_PARAM;        
+        LOG(log_error, logtype_cnid, "cnid_dbd_get: Parameter error");
+        errno = CNID_ERR_PARAM;
         return CNID_INVALID;
     }
 
     if (len > MAXPATHLEN) {
-        LOG(log_error, logtype_cnid, "cnid_add: Path name is too long");
+        LOG(log_error, logtype_cnid, "cnid_dbd_get: Path name is too long");
         errno = CNID_ERR_PATH;
         return CNID_INVALID;
     }
 
+    LOG(log_debug, logtype_cnid, "cnid_dbd_get: DID: %u, name: '%s'", ntohl(did), name);
+
     RQST_RESET(&rqst);
     rqst.op = CNID_DBD_OP_GET;
     rqst.did = did;
     rqst.name = name;
     rqst.namelen = len;
 
+    rply.namelen = 0;
     if (transmit(db, &rqst, &rply) < 0) {
         errno = CNID_ERR_DB;
         return CNID_INVALID;
     }
-    
+
     switch(rply.result) {
     case CNID_DBD_RES_OK:
         id = rply.cnid;
+        LOG(log_debug, logtype_cnid, "cnid_dbd_get: got CNID: %u", ntohl(id));
         break;
     case CNID_DBD_RES_NOTFOUND:
         id = CNID_INVALID;
@@ -547,7 +622,7 @@ cnid_t cnid_dbd_get(struct _cnid_db *cdb, const cnid_t did, const char *name,
         id = CNID_INVALID;
         errno = CNID_ERR_DB;
         break;
-    default: 
+    default:
         abort();
     }
 
@@ -555,7 +630,7 @@ cnid_t cnid_dbd_get(struct _cnid_db *cdb, const cnid_t did, const char *name,
 }
 
 /* ---------------------- */
-char *cnid_dbd_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, u_int32_t len)
+char *cnid_dbd_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, size_t len)
 {
     CNID_private *db;
     struct cnid_dbd_rqst rqst;
@@ -564,10 +639,12 @@ char *cnid_dbd_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, u_int32_t
 
     if (!cdb || !(db = cdb->_private) || !id || !(*id)) {
         LOG(log_error, logtype_cnid, "cnid_resolve: Parameter error");
-        errno = CNID_ERR_PARAM;                
+        errno = CNID_ERR_PARAM;
         return NULL;
     }
 
+    LOG(log_debug, logtype_cnid, "cnid_dbd_resolve: resolving CNID: %u", ntohl(*id));
+
     /* TODO: We should maybe also check len. At the moment we rely on the caller
        to provide a buffer that is large enough for MAXPATHLEN plus
        CNID_HEADER_LEN plus 1 byte, which is large enough for the maximum that
@@ -593,6 +670,7 @@ char *cnid_dbd_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, u_int32_t
     case CNID_DBD_RES_OK:
         *id = rply.did;
         name = rply.name;
+        LOG(log_debug, logtype_cnid, "cnid_dbd_resolve: resolved did: %u, name: '%s'", ntohl(*id), name);
         break;
     case CNID_DBD_RES_NOTFOUND:
         *id = CNID_INVALID;
@@ -610,47 +688,25 @@ char *cnid_dbd_resolve(struct _cnid_db *cdb, cnid_t *id, void *buffer, u_int32_t
     return name;
 }
 
-/* --------------------- */
-static int dbd_getstamp(CNID_private *db, void *buffer, const size_t len)
-{
-    struct cnid_dbd_rqst rqst;
-    struct cnid_dbd_rply rply;
-
-    memset(buffer, 0, len);
-    dbd_initstamp(&rqst);
-
-    rply.name = buffer;
-    rply.namelen = len;
-
-    if (transmit(db, &rqst, &rply) < 0) {
-        errno = CNID_ERR_DB;
-        return -1;
-    }
-    return dbd_reply_stamp(&rply);
-}
-
 /* ---------------------- */
-int cnid_dbd_getstamp(struct _cnid_db *cdb, void *buffer, const int len)
+int cnid_dbd_getstamp(struct _cnid_db *cdb, void *buffer, const size_t len)
 {
     CNID_private *db;
-    int ret;
 
     if (!cdb || !(db = cdb->_private) || len != ADEDLEN_PRIVSYN) {
         LOG(log_error, logtype_cnid, "cnid_getstamp: Parameter error");
-        errno = CNID_ERR_PARAM;                
+        errno = CNID_ERR_PARAM;
         return -1;
     }
-    ret = dbd_getstamp(db, buffer, len);
-    if (!ret) {
-    	db->notfirst = 1;
-    	memcpy(db->stamp, buffer, len);
-    }
-    return ret;
+    db->client_stamp = buffer;
+    db->stamp_size = len;
+    memset(buffer,0, len);
+    return 0;
 }
 
 /* ---------------------- */
 cnid_t cnid_dbd_lookup(struct _cnid_db *cdb, const struct stat *st, const cnid_t did,
-                   const char *name, const int len)
+                       char *name, const size_t len)
 {
     CNID_private *db;
     struct cnid_dbd_rqst rqst;
@@ -659,7 +715,7 @@ cnid_t cnid_dbd_lookup(struct _cnid_db *cdb, const struct stat *st, const cnid_t
 
     if (!cdb || !(db = cdb->_private) || !st || !name) {
         LOG(log_error, logtype_cnid, "cnid_lookup: Parameter error");
-        errno = CNID_ERR_PARAM;        
+        errno = CNID_ERR_PARAM;
         return CNID_INVALID;
     }
 
@@ -682,6 +738,10 @@ cnid_t cnid_dbd_lookup(struct _cnid_db *cdb, const struct stat *st, const cnid_t
     rqst.name = name;
     rqst.namelen = len;
 
+    LOG(log_debug, logtype_cnid, "cnid_dbd_lookup: CNID: %u, name: '%s', inode: 0x%llx, type: %d (0=file, 1=dir)",
+        ntohl(did), name, (long long)st->st_ino, rqst.type);
+
+    rply.namelen = 0;
     if (transmit(db, &rqst, &rply) < 0) {
         errno = CNID_ERR_DB;
         return CNID_INVALID;
@@ -690,6 +750,7 @@ cnid_t cnid_dbd_lookup(struct _cnid_db *cdb, const struct stat *st, const cnid_t
     switch (rply.result) {
     case CNID_DBD_RES_OK:
         id = rply.cnid;
+        LOG(log_debug, logtype_cnid, "cnid_dbd_lookup: got CNID: %u", ntohl(id));
         break;
     case CNID_DBD_RES_NOTFOUND:
         id = CNID_INVALID;
@@ -707,16 +768,15 @@ cnid_t cnid_dbd_lookup(struct _cnid_db *cdb, const struct stat *st, const cnid_t
 
 /* ---------------------- */
 int cnid_dbd_update(struct _cnid_db *cdb, const cnid_t id, const struct stat *st,
-                const cnid_t did, const char *name, const int len)
+                    const cnid_t did, char *name, const size_t len)
 {
     CNID_private *db;
     struct cnid_dbd_rqst rqst;
     struct cnid_dbd_rply rply;
 
-    
     if (!cdb || !(db = cdb->_private) || !id || !st || !name) {
         LOG(log_error, logtype_cnid, "cnid_update: Parameter error");
-        errno = CNID_ERR_PARAM;        
+        errno = CNID_ERR_PARAM;
         return -1;
     }
 
@@ -738,6 +798,10 @@ int cnid_dbd_update(struct _cnid_db *cdb, const cnid_t id, const struct stat *st
     rqst.name = name;
     rqst.namelen = len;
 
+    LOG(log_debug, logtype_cnid, "cnid_dbd_update: CNID: %u, name: '%s', inode: 0x%llx, type: %d (0=file, 1=dir)",
+        ntohl(id), name, (long long)st->st_ino, rqst.type);
+
+    rply.namelen = 0;
     if (transmit(db, &rqst, &rply) < 0) {
         errno = CNID_ERR_DB;
         return -1;
@@ -745,6 +809,7 @@ int cnid_dbd_update(struct _cnid_db *cdb, const cnid_t id, const struct stat *st
 
     switch (rply.result) {
     case CNID_DBD_RES_OK:
+        LOG(log_debug, logtype_cnid, "cnid_dbd_update: updated");
     case CNID_DBD_RES_NOTFOUND:
         return 0;
     case CNID_DBD_RES_ERR_DB:
@@ -756,23 +821,89 @@ int cnid_dbd_update(struct _cnid_db *cdb, const cnid_t id, const struct stat *st
 }
 
 /* ---------------------- */
-int cnid_dbd_delete(struct _cnid_db *cdb, const cnid_t id) 
+cnid_t cnid_dbd_rebuild_add(struct _cnid_db *cdb, const struct stat *st,
+                            const cnid_t did, char *name, const size_t len,
+                            cnid_t hint)
+{
+    CNID_private *db;
+    struct cnid_dbd_rqst rqst;
+    struct cnid_dbd_rply rply;
+    cnid_t id;
+
+    if (!cdb || !(db = cdb->_private) || !st || !name || hint == CNID_INVALID) {
+        LOG(log_error, logtype_cnid, "cnid_rebuild_add: Parameter error");
+        errno = CNID_ERR_PARAM;
+        return CNID_INVALID;
+    }
+
+    if (len > MAXPATHLEN) {
+        LOG(log_error, logtype_cnid, "cnid_rebuild_add: Path name is too long");
+        errno = CNID_ERR_PATH;
+        return CNID_INVALID;
+    }
+
+    RQST_RESET(&rqst);
+    rqst.op = CNID_DBD_OP_REBUILD_ADD;
+
+    if (!(cdb->flags & CNID_FLAG_NODEV)) {
+        rqst.dev = st->st_dev;
+    }
+
+    rqst.ino = st->st_ino;
+    rqst.type = S_ISDIR(st->st_mode)?1:0;
+    rqst.did = did;
+    rqst.name = name;
+    rqst.namelen = len;
+    rqst.cnid = hint;
+
+    LOG(log_debug, logtype_cnid, "cnid_dbd_rebuild_add: CNID: %u, name: '%s', inode: 0x%llx, type: %d (0=file, 1=dir), hint: %u",
+        ntohl(did), name, (long long)st->st_ino, rqst.type, hint);
+
+    if (transmit(db, &rqst, &rply) < 0) {
+        errno = CNID_ERR_DB;
+        return CNID_INVALID;
+    }
+
+    switch(rply.result) {
+    case CNID_DBD_RES_OK:
+        id = rply.cnid;
+        LOG(log_debug, logtype_cnid, "cnid_dbd_rebuild_add: got CNID: %u", ntohl(id));
+        break;
+    case CNID_DBD_RES_ERR_MAX:
+        errno = CNID_ERR_MAX;
+        id = CNID_INVALID;
+        break;
+    case CNID_DBD_RES_ERR_DB:
+    case CNID_DBD_RES_ERR_DUPLCNID:
+        errno = CNID_ERR_DB;
+        id = CNID_INVALID;
+        break;
+    default:
+        abort();
+    }
+    return id;
+}
+
+/* ---------------------- */
+int cnid_dbd_delete(struct _cnid_db *cdb, const cnid_t id)
 {
     CNID_private *db;
     struct cnid_dbd_rqst rqst;
     struct cnid_dbd_rply rply;
 
-
     if (!cdb || !(db = cdb->_private) || !id) {
         LOG(log_error, logtype_cnid, "cnid_delete: Parameter error");
-        errno = CNID_ERR_PARAM;        
+        errno = CNID_ERR_PARAM;
         return -1;
     }
 
+    LOG(log_debug, logtype_cnid, "cnid_dbd_delete: delete CNID: %u", ntohl(id));
+
     RQST_RESET(&rqst);
     rqst.op = CNID_DBD_OP_DELETE;
     rqst.cnid = id;
 
+    rply.namelen = 0;
     if (transmit(db, &rqst, &rply) < 0) {
         errno = CNID_ERR_DB;
         return -1;
@@ -780,6 +911,7 @@ int cnid_dbd_delete(struct _cnid_db *cdb, const cnid_t id)
 
     switch (rply.result) {
     case CNID_DBD_RES_OK:
+        LOG(log_debug, logtype_cnid, "cnid_dbd_delete: deleted CNID: %u", ntohl(id));
     case CNID_DBD_RES_NOTFOUND:
         return 0;
     case CNID_DBD_RES_ERR_DB:
@@ -790,10 +922,12 @@ int cnid_dbd_delete(struct _cnid_db *cdb, const cnid_t id)
     }
 }
 
+
 struct _cnid_module cnid_dbd_module = {
     "dbd",
     {NULL, NULL},
     cnid_dbd_open,
+    0
 };
 
 #endif /* CNID_DBD */
