@@ -1,5 +1,5 @@
 /*
- * $Id: dsi_stream.c,v 1.11.6.5.2.1 2005/01/31 19:50:53 didg Exp $
+ * $Id: dsi_stream.c,v 1.20 2009-10-26 12:35:56 franklahm Exp $
  *
  * Copyright (c) 1998 Adrian Sun (asun@zoology.washington.edu)
  * All rights reserved. See COPYRIGHT.
@@ -34,9 +34,9 @@
 #endif
 
 #include <atalk/logger.h>
-
 #include <atalk/dsi.h>
 #include <netatalk/endian.h>
+#include <atalk/util.h>
 
 #define min(a,b)  ((a) < (b) ? (a) : (b))
 
@@ -51,12 +51,42 @@
 /* ------------------------- 
  * we don't use a circular buffer.
 */
-void dsi_buffer(DSI *dsi)
+static void dsi_init_buffer(DSI *dsi)
+{
+    if (!dsi->buffer) {
+        /* XXX config options */
+        dsi->maxsize = 6 * dsi->server_quantum;
+        if (!dsi->maxsize)
+            dsi->maxsize = 6 * DSI_SERVQUANT_DEF;
+        dsi->buffer = malloc(dsi->maxsize);
+        if (!dsi->buffer) {
+            return;
+        }
+        dsi->start = dsi->buffer;
+        dsi->eof = dsi->buffer;
+        dsi->end = dsi->buffer + dsi->maxsize;
+    }
+}
+
+/* ---------------------- 
+   afpd is sleeping too much while trying to send something.
+   May be there's no reader or the reader is also sleeping in write,
+   look if there's some data for us to read, hopefully it will wake up
+   the reader
+*/
+static int dsi_buffer(DSI *dsi)
 {
     fd_set readfds, writefds;
     int    len;
     int    maxfd;
 
+    /* non blocking mode */
+    if (setnonblock(dsi->socket, 1) < 0) {
+        /* can't do it! exit without error it will sleep to death below */
+        LOG(log_error, logtype_default, "dsi_buffer: ioctl non blocking mode %s", strerror(errno));
+        return 0;
+    }
+    
     FD_ZERO(&readfds);
     FD_ZERO(&writefds);
     FD_SET( dsi->socket, &readfds);
@@ -66,27 +96,13 @@ void dsi_buffer(DSI *dsi)
         FD_SET( dsi->socket, &readfds);
         FD_SET( dsi->socket, &writefds);
         if (select( maxfd, &readfds, &writefds, NULL, NULL) <= 0)
-            return;
+            break;
 
         if ( !FD_ISSET(dsi->socket, &readfds)) {
             /* nothing waiting in the read queue */
-            return;
-        }
-        if (!dsi->buffer) {
-            /* XXX config options */
-            dsi->maxsize = 6 * dsi->server_quantum;
-            if (!dsi->maxsize)
-                dsi->maxsize = 6 * DSI_SERVQUANT_DEF;
-            dsi->buffer = malloc(dsi->maxsize);
-            if (!dsi->buffer) {
-                /* fall back to blocking IO */
-                dsi_block(dsi, 0);
-                return;
-            }
-            dsi->start = dsi->buffer;
-            dsi->eof = dsi->buffer;
-            dsi->end = dsi->buffer + dsi->maxsize;
+            break;
         }
+        dsi_init_buffer(dsi);
         len = dsi->end - dsi->eof;
 
         if (len <= 0) {
@@ -94,25 +110,31 @@ void dsi_buffer(DSI *dsi)
              * fall back to blocking IO 
              * could block and disconnect but it's better than a cpu hog
              */
-            dsi_block(dsi, 0);
-            return;
+            break;
         }
 
         len = read(dsi->socket, dsi->eof, len);
         if (len <= 0)
-            return;
+            break;
         dsi->eof += len;
         if ( FD_ISSET(dsi->socket, &writefds)) {
-            return;
+            /* we can write again at last */
+            break;
         }
     }
+    if (setnonblock(dsi->socket, 0) < 0) {
+        /* can't do it! afpd will fail very quickly */
+        LOG(log_error, logtype_default, "dsi_buffer: ioctl blocking mode %s", strerror(errno));
+        return -1;
+    }
+    return 0;
 }
 
 /* ------------------------------
  * write raw data. return actual bytes read. checks against EINTR
  * aren't necessary if all of the signals have SA_RESTART
  * specified. */
-size_t dsi_stream_write(DSI *dsi, void *data, const size_t length, int mode)
+ssize_t dsi_stream_write(DSI *dsi, void *data, const size_t length, int mode)
 {
   size_t written;
   ssize_t len;
@@ -125,11 +147,10 @@ size_t dsi_stream_write(DSI *dsi, void *data, const size_t length, int mode)
 #if 0
   /* XXX there's no MSG_DONTWAIT in recv ?? so we have to play with ioctl
   */ 
-  if (dsi->noblocking) {
-      flags |= MSG_DONTWAIT;
-  }
+  flags |= MSG_DONTWAIT;
 #endif
   
+  dsi->in_write++;
   written = 0;
   while (written < length) {
     if ((-1 == (len = send(dsi->socket, (u_int8_t *) data + written,
@@ -138,17 +159,23 @@ size_t dsi_stream_write(DSI *dsi, void *data, const size_t length, int mode)
       continue;
 
     if (len < 0) {
-      if (dsi->noblocking && errno ==  EAGAIN) {
-         /* non blocking mode but will block 
-          * read data in input queue.
-          * 
-         */
-         dsi_buffer(dsi);
-      }
-      else {
-          LOG(log_error, logtype_default, "dsi_stream_write: %s", strerror(errno));
-          break;
+      if (errno == EAGAIN || errno == EWOULDBLOCK) {
+          if (mode == DSI_NOWAIT && written == 0) {
+              /* DSI_NOWAIT is used by attention
+                 give up in this case.
+              */
+              return -1;
+          }
+          if (dsi_buffer(dsi)) {
+              /* can't go back to blocking mode, exit, the next read
+                 will return with an error and afpd will die.
+              */
+              break;
+          }
+          continue;
       }
+      LOG(log_error, logtype_default, "dsi_stream_write: %s", strerror(errno));
+      break;
     }
     else {
         written += len;
@@ -156,19 +183,64 @@ size_t dsi_stream_write(DSI *dsi, void *data, const size_t length, int mode)
   }
 
   dsi->write_count += written;
+  dsi->in_write--;
   return written;
 }
 
+
 /* ---------------------------------
 */
-static ssize_t buf_read(DSI *dsi, u_int8_t *buf, size_t count)
+#ifdef WITH_SENDFILE
+ssize_t dsi_stream_read_file(DSI *dsi, int fromfd, off_t offset, const size_t length)
 {
-    ssize_t nbe = 0;
-    ssize_t ret;
-    
-    if (!count)
-        return 0;
+  size_t written;
+  ssize_t len;
+
+  dsi->in_write++;
+  written = 0;
+
+  while (written < length) {
+    len = sys_sendfile(dsi->socket, fromfd, &offset, length - written);
         
+    if (len < 0) {
+      if (errno == EINTR)
+          continue;
+      if (errno == EINVAL || errno == ENOSYS)
+          return -1;
+          
+      if (errno == EAGAIN || errno == EWOULDBLOCK) {
+          if (dsi_buffer(dsi)) {
+              /* can't go back to blocking mode, exit, the next read
+                 will return with an error and afpd will die.
+              */
+              break;
+          }
+          continue;
+      }
+      LOG(log_error, logtype_default, "dsi_stream_write: %s", strerror(errno));
+      break;
+    }
+    else if (!len) {
+        /* afpd is going to exit */
+        errno = EIO;
+        return -1; /* I think we're at EOF here... */
+    }
+    else 
+        written += len;
+  }
+
+  dsi->write_count += written;
+  dsi->in_write--;
+  return written;
+}
+#endif
+
+/* ---------------------------------
+*/
+static size_t from_buf(DSI *dsi, u_int8_t *buf, size_t count)
+{
+    size_t nbe = 0;
+    
     if (dsi->start) {        
         nbe = dsi->eof - dsi->start;
 
@@ -180,20 +252,24 @@ static ssize_t buf_read(DSI *dsi, u_int8_t *buf, size_t count)
            if (dsi->eof == dsi->start) 
                dsi->start = dsi->eof = dsi->buffer;
 
-           if (nbe == count)
-               return nbe;
-           count -= nbe;
-           buf += nbe;
         }
-        else 
-           nbe = 0;
     }
+    return nbe;
+}
+
+static ssize_t buf_read(DSI *dsi, u_int8_t *buf, size_t count)
+{
+    ssize_t nbe;
+    
+    if (!count)
+        return 0;
+
+    nbe = from_buf(dsi, buf, count);
+    if (nbe)
+        return nbe;
   
-    ret = read(dsi->socket, buf, count);
-    if (ret <= 0)
-        return ret;
+    return read(dsi->socket, buf, count);
 
-    return ret +nbe;
 }
 
 /* ---------------------------------------
@@ -225,6 +301,32 @@ size_t dsi_stream_read(DSI *dsi, void *data, const size_t length)
   return stored;
 }
 
+/* ---------------------------------------
+ * read raw data. return actual bytes read. this will wait until 
+ * it gets length bytes 
+ */
+static size_t dsi_buffered_stream_read(DSI *dsi, u_int8_t *data, const size_t length)
+{
+  size_t len;
+  size_t buflen;
+  
+  dsi_init_buffer(dsi);
+  len = from_buf(dsi, data, length);
+  dsi->read_count += len;
+  if (len == length) {
+      return len;
+  }
+  
+  buflen = min(8192, dsi->end - dsi->eof);
+  if (buflen > 0) {
+      ssize_t ret;
+      ret = read(dsi->socket, dsi->eof, buflen);
+      if (ret > 0)
+          dsi->eof += ret;
+  }
+  return dsi_stream_read(dsi, data, length -len);
+}
+
 /* ---------------------------------------
 */
 void dsi_sleep(DSI *dsi, const int state)
@@ -236,14 +338,14 @@ void dsi_sleep(DSI *dsi, const int state)
 */
 static void block_sig(DSI *dsi)
 {
-  if (!dsi->sigblocked) sigprocmask(SIG_BLOCK, &dsi->sigblockset, &dsi->oldset);
+  dsi->in_write++;
 }
 
 /* ---------------------------------------
 */
 static void unblock_sig(DSI *dsi)
 {
-  if (!dsi->sigblocked) sigprocmask(SIG_SETMASK, &dsi->oldset, NULL);
+  dsi->in_write--;
 }
 
 /* ---------------------------------------
@@ -288,9 +390,14 @@ int dsi_stream_send(DSI *dsi, void *buf, size_t length)
 	!len)
       continue;
     
-    if (len == towrite) /* wrote everything out */
+    if ((size_t)len == towrite) /* wrote everything out */
       break;
     else if (len < 0) { /* error */
+      if (errno == EAGAIN || errno == EWOULDBLOCK) {
+          if (!dsi_buffer(dsi)) {
+              continue;
+          }
+      }
       LOG(log_error, logtype_default, "dsi_stream_send: %s", strerror(errno));
       unblock_sig(dsi);
       return 0;
@@ -334,7 +441,7 @@ int dsi_stream_receive(DSI *dsi, void *buf, const size_t ilength,
   char block[DSI_BLOCKSIZ];
 
   /* read in the header */
-  if (dsi_stream_read(dsi, block, sizeof(block)) != sizeof(block)) 
+  if (dsi_buffered_stream_read(dsi, (u_int8_t *)block, sizeof(block)) != sizeof(block)) 
     return 0;
 
   dsi->header.dsi_flags = block[0];
